@@ -17,29 +17,21 @@ sub preprocess (@) { #{{{
 	my ($image) = $_[0] =~ /$config{wiki_file_regexp}/; # untaint
 	my %params=@_;
 
-	if (exists $imgdefaults{$params{page}}) {
-		foreach my $key (keys %{$imgdefaults{$params{page}}}) {
-			if (! exists $params{$key}) {
-				$params{$key}=$imgdefaults{$params{page}}->{$key};
-			}
-		}
-	}
-
-	if (! exists $params{size}) {
-		$params{size}='full';
+	if (! exists $imgdefaults{$params{page}}) {
+		$imgdefaults{$params{page}} = {};
 	}
+	my $size = $params{size} || $imgdefaults{$params{page}}->{size} || 'full';
+	my $alt = $params{alt} || $imgdefaults{$params{page}}->{alt} || '';
 
 	if ($image eq 'defaults') {
-		$imgdefaults{$params{page}} = \%params;
+		$imgdefaults{$params{page}} = {
+			size => $size,
+			alt => $alt,
+		};
 		return '';
 	}
 
 	push @{$links{$params{page}}}, $image;
-	# optimisation: detect scan mode, and avoid generating the image
-	if (! defined wantarray) {
-		return;
-	}
-
 	my $file = bestlink($params{page}, $image);
 
 	my $dir = $params{page};
@@ -51,11 +43,9 @@ sub preprocess (@) { #{{{
 	my $imglink;
 	my $r;
 
-	my ($dwidth, $dheight);
-
-	if ($params{size} ne 'full') {
-		my ($w, $h) = ($params{size} =~ /^(\d+)x(\d+)$/);
-		return "[[img ".sprintf(gettext('bad size "%s"'), $params{size})."]]"
+	if ($size ne 'full') {
+		my ($w, $h) = ($size =~ /^(\d+)x(\d+)$/);
+		return "[[img ".sprintf(gettext('bad size "%s"'), $size)."]]"
 			unless (defined $w && defined $h);
 
 		my $outfile = "$config{destdir}/$dir/${w}x${h}-$base";
@@ -71,15 +61,7 @@ sub preprocess (@) { #{{{
 			$r = $im->Read(srcfile($file));
 			return "[[img ".sprintf(gettext("failed to read %s: %s"), $file, $r)."]]" if $r;
 
-			# don't resize any larger
-			my ($rw, $rh) = ($w, $h);
-			if ($rw > $im->Get("width") ||
-			    $rh > $im->Get("height")) {
-				$rw=$im->Get("width");
-				$rh=$im->Get("height");
-			}
-
-			$r = $im->Resize(geometry => "${rw}x${rh}");
+			$r = $im->Resize(geometry => "${w}x${h}");
 			return "[[img ".sprintf(gettext("failed to resize: %s"), $r)."]]" if $r;
 
 			# don't actually write file in preview mode
@@ -91,17 +73,11 @@ sub preprocess (@) { #{{{
 				$imglink = $file;
 			}
 		}
-
-		# since we don't really resize larger, set the display
-		# size, so the browser can scale the image up if necessary
-		($dwidth, $dheight)=($w, $h);
 	}
 	else {
 		$r = $im->Read(srcfile($file));
 		return "[[img ".sprintf(gettext("failed to read %s: %s"), $file, $r)."]]" if $r;
 		$imglink = $file;
-		$dwidth = $im->Get("width");
-		$dheight = $im->Get("height");
 	}
 
 	add_depends($imglink, $params{page});
@@ -120,37 +96,12 @@ sub preprocess (@) { #{{{
 		return "[[img ".sprintf(gettext("failed to determine size of image %s"), $file)."]]";
 	}
 
-	my $imgtag='<img src="'.$imgurl.
-		'" alt="'.(exists $params{alt} ? $params{alt} : '').
-		'" width="'.$dwidth.
-		'" height="'.$dheight.'"'.
-		(exists $params{title} ? ' title="'.$params{title}.'"' : '').
+	return '<a href="'.$fileurl.'"><img src="'.$imgurl.
+		'" alt="'.$alt.'" width="'.$im->Get("width").
+		'" height="'.$im->Get("height").'"'.
 		(exists $params{class} ? ' class="'.$params{class}.'"' : '').
 		(exists $params{id} ? ' id="'.$params{id}.'"' : '').
-		' />';
-
-	if (! defined $params{link} || lc($params{link}) eq 'yes') {
-		$imgtag='<a href="'.$fileurl.'">'.$imgtag.'</a>';
-	}
-	elsif ($params{link} =~ /^\w+:\/\//) {
-		$imgtag='<a href="'.$params{link}.'">'.$imgtag.'</a>';
-	}
-	elsif (length bestlink($params{page}, $params{link})) {
-		add_depends($params{page}, $params{link});
-		$imgtag=htmllink($params{page}, $params{destpage},
-			$params{link}, linktext => $imgtag,
-			noimageinline => 1);
-	}
-
-	if (exists $params{caption}) {
-		return '<table class="img">'.
-			'<caption>'.$params{caption}.'</caption>'.
-			'<tr><td>'.$imgtag.'</td></tr>'.
-			'</table>';
-	}
-	else {
-		return $imgtag;
-	}
+		' /></a>';
 } #}}}
 
 1
