@@ -27,22 +27,22 @@ sub formbuilder_setup (@) { #{{{
 	my $session=$params{session};
 	my $cgi=$params{cgi};
 	
-	if ($form->title eq "signin") {
-		# Give up if module is unavailable to avoid
-		# needing to depend on it.
-		eval q{use Net::OpenID::Consumer};
-		if ($@) {
-			debug("unable to load Net::OpenID::Consumer, not enabling OpenID login");
-			return;
-		}
+	# Give up if module is unavailable to avoid needing to depend on
+	# it.
+	eval q{use Net::OpenID::Consumer};
+	if ($@) {
+		debug("unable to load Net::OpenID::Consumer, not enabling OpenID login");
+		return;
+	}
 
+	if ($form->title eq "signin") {
 		# This avoids it displaying a redundant label for the
 		# OpenID fieldset.
 		$form->fieldsets("OpenID");
 
  		$form->field(
 			name => "openid_url",
-			label => gettext("Log in with")." ".htmllink("", "", "ikiwiki/OpenID", noimageinline => 1),
+			label => gettext("Log in with")." ".htmllink("", "", "OpenID", noimageinline => 1),
 			fieldset => "OpenID",
 			size => 30,
 			comment => ($config{openidsignup} ? " | <a href=\"$config{openidsignup}\">".gettext("Get an OpenID")."</a>" : "")
@@ -164,31 +164,4 @@ sub getobj ($$) { #{{{
 	);
 } #}}}
 
-package IkiWiki;
-
-# This is not used by this plugin, but this seems the best place to put it.
-# Used elsewhere to pretty-display the name of an openid user.
-sub openiduser ($) { #{{{
-	my $user=shift;
-
-	if ($user =~ m!^https?://! &&
-	    eval q{use Net::OpenID::VerifiedIdentity; 1} && !$@) {
-		my $oid=Net::OpenID::VerifiedIdentity->new(identity => $user);
-		my $display=$oid->display;
-		# Convert "user.somehost.com" to "user [somehost.com]".
-		if ($display !~ /\[/) {
-			$display=~s/^(.*?)\.([^.]+\.[a-z]+)$/$1 [$2]/;
-		}
-		# Convert "http://somehost.com/user" to "user [somehost.com]".
-		if ($display !~ /\[/) {
-			$display=~s/^https?:\/\/(.+)\/([^\/]+)$/$2 [$1]/;
-		}
-		$display=~s!^https?://!!; # make sure this is removed
-		eval q{use CGI 'escapeHTML'};
-		error($@) if $@;
-		return escapeHTML($display);
-	}
-	return;
-}
-
 1
