@@ -27,7 +27,7 @@ sub sidebar_content ($) { #{{{
 
 		my $content=readfile(srcfile($sidebar_file));
 		return unless length $content;
-		return IkiWiki::htmlize($sidebar_page, $page, $sidebar_type,
+		return IkiWiki::htmlize($page, $sidebar_type,
 		       IkiWiki::linkify($sidebar_page, $page,
 		       IkiWiki::preprocess($sidebar_page, $page,
 		       IkiWiki::filter($sidebar_page, $page, $content))));
