@@ -46,15 +46,7 @@ sub preprocess (@) { #{{{
 	}
 
 	foreach my $param (keys %params) {
-		if ($template->query(name => $param)) {
-			$template->param($param =>
-				IkiWiki::htmlize($params{page}, $params{destpage},
-					pagetype($pagesources{$params{page}}),
-					$params{$param}));
-		}
-		if ($template->query(name => "raw_$param")) {
-			$template->param("raw_$param" => $params{$param});
-		}
+		$template->param($param => $params{$param});
 	}
 
 	return IkiWiki::preprocess($params{page}, $params{destpage},
