@@ -9,7 +9,7 @@ use IkiWiki 2.00;
 # of css to hide toggleables, to avoid any flashing on page load. The css
 # is only emitted after the javascript tests that it's going to be able to
 # show the toggleables.
-our $javascript=<<'EOF';
+my $javascript=<<'EOF';
 <script type="text/javascript">
 <!--
 if (document.getElementById && document.getElementsByTagName && document.createTextNode) {
@@ -21,8 +21,7 @@ function inittoggle() {
 	var as = getElementsByClass('toggle');
 	for (var i = 0; i < as.length; i++) {
 		var id = as[i].href.match(/#(\w.+)/)[1];
-		if (document.getElementById(id).className == "toggleable")
-			document.getElementById(id).style.display="none";
+		document.getElementById(id).style.display="none";
 		as[i].onclick = function() {
 			toggle(this);
 			return false;
@@ -39,9 +38,9 @@ function toggle(s) {
 		style.display = "none";
 }
 
-function getElementsByClass(c) {
+function getElementsByClass(class) {
 	var ret = new Array();
-	var pattern = new RegExp("(^|\\s)"+c+"(\\s|$)");
+	var pattern = new RegExp("(^|\\s)"+class+"(\\s|$)");
 	var els = document.getElementsByTagName('*');
 	for (i = 0, j = 0; i < els.length; i++) {
 		if ( pattern.test(els[i].className) ) {
@@ -85,7 +84,7 @@ sub preprocess_toggle (@) { #{{{
 } # }}}
 
 sub preprocess_toggleable (@) { #{{{
-	my %params=(id => "default", text => "", open => "no", @_);
+	my %params=(id => "default", text => "", @_);
 
 	# Preprocess the text to expand any preprocessor directives
 	# embedded inside it.
@@ -93,25 +92,21 @@ sub preprocess_toggleable (@) { #{{{
 		IkiWiki::filter($params{page}, $params{destpage}, $params{text}));
 	
 	my $id=genid($params{page}, $params{id});
-	my $class=(lc($params{open}) ne "yes") ? "toggleable" : "toggleable-open";
 
 	# Should really be a postprocessor directive, oh well. Work around
 	# markdown's dislike of markdown inside a <div> with various funky
 	# whitespace.
 	my ($indent)=$params{text}=~/( +)$/;
 	$indent="" unless defined $indent;
-	return "<div class=\"$class\" id=\"$id\"></div>\n\n$params{text}\n$indent<div class=\"toggleableend\"></div>";
+	return "<div class=\"toggleable\" id=\"$id\"></div>\n\n$params{text}\n$indent<div class=\"toggleableend\"></div>";
 } # }}}
 
 sub format (@) { #{{{
         my %params=@_;
 
-	if ($params{content}=~s!(<div class="toggleable(?:-open)?" id="[^"]+">\s*)</div>!$1!g) {
+	if ($params{content}=~s!(<div class="toggleable" id="[^"]+">)</div>!$1!g) {
 		$params{content}=~s/<div class="toggleableend">//g;
-		if (! ($params{content}=~s!^<body>!<body>$javascript!m)) {
-			# no </body> tag, probably in preview mode
-			$params{content}=$javascript.$params{content};
-		}
+		$params{content}=~s!^<\/body>!$javascript</body>!m;
 	}
 	return $params{content};
 } # }}}
