@@ -29,15 +29,12 @@ sub rcs_commit ($$$;$$) {
 
 sub rcs_add ($) {
 	# Add a file. The filename is relative to the root of the srcdir.
-	# Note that this should not check the new file in, it should only
-	# prepare for it to be checked in when rcs_commit is called.
 }
 
 sub rcs_recentchanges ($) {
 	# Examine the RCS history and generate a list of recent changes.
 	# The data structure returned for each change is:
 	# {
-	# 	rev => # the RCSs id for this commit
 	# 	user => # name of user who made the change,
 	# 	committype => # either "web" or the name of the rcs,
 	# 	when => # time when the change was made,
@@ -57,11 +54,11 @@ sub rcs_recentchanges ($) {
 	# }
 }
 
-sub rcs_diff ($) {
-	# Optional, used to get diffs for recentchanges.
-	# The parameter is the rev from rcs_recentchanges.
-	# Should return a list of lines of the diff (including \n) in list
-	# context, and the whole diff in scalar context.
+sub rcs_notify () {
+	# This function is called when a change is committed to the wiki,
+	# and ikiwiki is running as a post-commit hook from the RCS.
+	# It should examine the repository to somehow determine what pages
+	# changed, and then send emails to users subscribed to those pages.
 }
 
 sub rcs_getctime ($) {
