@@ -104,8 +104,10 @@ sub rcs_add ($) { # {{{
 sub rcs_recentchanges ($) { #{{{
 	my ($num) = @_;
 
-	my @cmdline = ("hg", "-R", $config{srcdir}, "log", "-v", "-l", $num,
-		"--style", "default");
+	eval q{use CGI 'escapeHTML'};
+	error($@) if $@;
+
+	my @cmdline = ("hg", "-R", $config{srcdir}, "log", "-v", "-l", $num);
 	open (my $out, "@cmdline |");
 
 	eval q{use Date::Parse};
@@ -139,7 +141,7 @@ sub rcs_recentchanges ($) { #{{{
 			rev        => $info->{"changeset"},
 			user       => $user,
 			committype => "mercurial",
-			when       => str2time($info->{"date"}),
+			when       => time - str2time($info->{"date"}),
 			message    => [@message],
 			pages      => [@pages],
 		};
@@ -148,7 +150,7 @@ sub rcs_recentchanges ($) { #{{{
 	return @ret;
 } #}}}
 
-sub rcs_diff ($) { #{{{
+sub rcs_notify () { #{{{
 	# TODO
 } #}}}
 
@@ -157,8 +159,7 @@ sub rcs_getctime ($) { #{{{
 
 	# XXX filename passes through the shell here, should try to avoid
 	# that just in case
-	my @cmdline = ("hg", "-R", $config{srcdir}, "log", "-v", "-l", '1', 
-		"--style", "default", "$config{srcdir}/$file");
+	my @cmdline = ("hg", "-R", $config{srcdir}, "log", "-v", "-l", '1', $file);
 	open (my $out, "@cmdline |");
 
 	my @log = mercurial_log($out);
