@@ -171,7 +171,7 @@ sub rcs_recentchanges ($) { #{{{
 		my $rev = $logentry->{revision};
 		my $user = $logentry->{author};
 
-		my $when=str2time($logentry->{date}, 'UTC');
+		my $when=time - str2time($logentry->{date}, 'UTC');
 
 		foreach my $msgline (split(/\n/, $logentry->{msg})) {
 			push @message, { line => $msgline };
@@ -187,24 +187,20 @@ sub rcs_recentchanges ($) { #{{{
 			$committype="svn";
 		}
 
-		foreach my $file (keys %{$logentry->{paths}}) {
-			if (length $config{svnpath}) {
-				next unless $file=~/^\/\Q$config{svnpath}\E\/([^ ]+)(?:$|\s)/;
-				$file=$1;
-			}
-
+		foreach (keys %{$logentry->{paths}}) {
+			next unless ! length $config{svnpath} || 
+			            /^\/\Q$config{svnpath}\E\/([^ ]+)(?:$|\s)/;
+			my $file=$1;
 			my $diffurl=$config{diffurl};
 			$diffurl=~s/\[\[file\]\]/$file/g;
 			$diffurl=~s/\[\[r1\]\]/$rev - 1/eg;
 			$diffurl=~s/\[\[r2\]\]/$rev/g;
-
 			push @pages, {
 				page => pagename($file),
 				diffurl => $diffurl,
 			} if length $file;
 		}
-		push @ret, {
-			rev => $rev,
+		push @ret, { rev => $rev,
 			user => $user,
 			committype => $committype,
 			when => $when,
@@ -217,9 +213,38 @@ sub rcs_recentchanges ($) { #{{{
 	return @ret;
 } #}}}
 
-sub rcs_diff ($) { #{{{
-	my $rev=possibly_foolish_untaint(int(shift));
-	return `svnlook diff $config{svnrepo} -r$rev --no-diff-deleted`;
+sub rcs_notify () { #{{{
+	if (! exists $ENV{REV}) {
+		error(gettext("REV is not set, not running from svn post-commit hook, cannot send notifications"));
+	}
+	my $rev=int(possibly_foolish_untaint($ENV{REV}));
+	
+	my $user=`svnlook author $config{svnrepo} -r $rev`;
+	chomp $user;
+	
+	my $message=`svnlook log $config{svnrepo} -r $rev`;
+	if ($message=~/$config{web_commit_regexp}/) {
+		$user=defined $2 ? "$2" : "$3";
+		$message=$4;
+	}
+
+	my @changed_pages;
+	foreach my $change (`svnlook changed $config{svnrepo} -r $rev`) {
+		chomp $change;
+		if (! length $config{svnpath} || 
+		    $change =~ /^[A-Z]+\s+\Q$config{svnpath}\E\/(.*)/) {
+			push @changed_pages, $1;
+		}
+	}
+	
+	require IkiWiki::UserInfo;
+	send_commit_mails(
+		sub {
+			return $message;
+		},
+		sub {
+			`svnlook diff $config{svnrepo} -r $rev --no-diff-deleted`;
+		}, $user, @changed_pages);
 } #}}}
 
 sub rcs_getctime ($) { #{{{
