@@ -3,13 +3,16 @@
 use warnings;
 use strict;
 use IkiWiki;
+use open qw{:utf8 :std};
 
 package IkiWiki;
 
 sub setup () { # {{{
 	my $setup=possibly_foolish_untaint($config{setup});
 	delete $config{setup};
-	open (IN, $setup) || error("read $setup: $!\n");
+	#translators: The first parameter is a filename, and the second
+	#translators: is a (probably not translated) error message.
+	open (IN, $setup) || error(sprintf(gettext("cannot read %s: %s"), $setup, $!));
 	my $code;
 	{
 		local $/=undef;
