@@ -63,25 +63,20 @@ sub setup_standard {
 			$config{$c}=undef;
 		}
 	}
-	
-	if (! $config{refresh}) {
-		$config{rebuild}=1;
-	}
-	
-	loadplugins();
-	checkconfig();
 
 	if ($config{render}) {
 		commandline_render();
 	}
-
-	if (! $config{refresh}) {
+	elsif (! $config{refresh}) {
+		$config{rebuild}=1;
 		debug(gettext("rebuilding wiki.."));
 	}
 	else {
 		debug(gettext("refreshing wiki.."));
 	}
 
+	loadplugins();
+	checkconfig();
 	lockwiki();
 	loadindex();
 	refresh();
