@@ -13,22 +13,22 @@ sub gen_wrapper () { #{{{
 	$config{destdir}=abs_path($config{destdir});
 	my $this=abs_path($0);
 	if (! -x $this) {
-		error("$this doesn't seem to be executable");
+		error(sprintf(gettext("%s doesn't seem to be executable"), $this));
 	}
 
 	if ($config{setup}) {
-		error("cannot create a wrapper that uses a setup file");
+		error(gettext("cannot create a wrapper that uses a setup file"));
 	}
 	my $wrapper=possibly_foolish_untaint($config{wrapper});
 	if (! defined $wrapper || ! length $wrapper) {
-		error("wrapper filename not specified");
+		error(gettext("wrapper filename not specified"));
 	}
 	delete $config{wrapper};
 	
 	my @envsave;
 	push @envsave, qw{REMOTE_ADDR QUERY_STRING REQUEST_METHOD REQUEST_URI
 	               CONTENT_TYPE CONTENT_LENGTH GATEWAY_INTERFACE
-		       HTTP_COOKIE} if $config{cgi};
+		       HTTP_COOKIE REMOTE_USER} if $config{cgi};
 	my $envsave="";
 	foreach my $var (@envsave) {
 		$envsave.=<<"EOF"
@@ -46,18 +46,27 @@ EOF
 		asprintf(&newenviron[i++], "%s=%s", "REV", s);
 EOF
 	}
+	if ($config{rcs} eq "tla" && $config{notify}) {
+		$envsave.=<<"EOF"
+	if ((s=getenv("ARCH_VERSION")))
+		asprintf(&newenviron[i++], "%s=%s", "ARCH_VERSION", s);
+EOF
+	}
 	
 	$Data::Dumper::Indent=0; # no newlines
 	my $configstring=Data::Dumper->Dump([\%config], ['*config']);
 	$configstring=~s/\\/\\\\/g;
 	$configstring=~s/"/\\"/g;
-	$configstring=~s/\n/\\\n/g;
+	$configstring=~s/\n/\\n/g;
 	
-	open(OUT, ">$wrapper.c") || error("failed to write $wrapper.c: $!");;
+	#translators: The first parameter is a filename, and the second is
+	#translators: a (probably not translated) error message.
+	open(OUT, ">$wrapper.c") || error(sprintf(gettext("failed to write %s: %s"), "$wrapper.c", $!));;
 	print OUT <<"EOF";
 /* A wrapper for ikiwiki, can be safely made suid. */
 #define _GNU_SOURCE
 #include <stdio.h>
+#include <sys/types.h>
 #include <unistd.h>
 #include <stdlib.h>
 #include <string.h>
@@ -75,6 +84,11 @@ $envsave
 	newenviron[i]=NULL;
 	environ=newenviron;
 
+	if (setregid(getegid(), -1) != 0 || setreuid(geteuid(), -1) != 0) {
+		perror("failed to drop real uid/gid");
+		exit(1);
+	}
+
 	execl("$this", "$this", NULL);
 	perror("failed to run $this");
 	exit(1);
@@ -82,14 +96,17 @@ $envsave
 EOF
 	close OUT;
 	if (system("gcc", "$wrapper.c", "-o", $wrapper) != 0) {
-		error("failed to compile $wrapper.c");
+		#translators: The parameter is a C filename.
+		error(sprintf(gettext("failed to compile %s"), "$wrapper.c"));
 	}
 	unlink("$wrapper.c");
 	if (defined $config{wrappermode} &&
 	    ! chmod(oct($config{wrappermode}), $wrapper)) {
 		error("chmod $wrapper: $!");
 	}
-	print "successfully generated $wrapper\n";
+	#translators: The parameter is a filename.
+	printf(gettext("successfully generated %s"), $wrapper);
+	print "\n";
 } #}}}
 
 1
